program mcforQ , rclass
version 8.2
*! Tom: 5 May 2005; only for logRR (Mantel-Haenszel) allowing variation of RR_MH
*! version 1.5 
syntax varlist(min=4 max=4 numeric) [if] [in]  [, save(string) sims(integer 10000)  ///
	CCOne(real 0.5) CCBoth(real 0.5) novaryrr novaryprev ///
	seed(integer -999) noVERBose ]

/* The whole process should be run quietly */
quietly {  

	noi di as txt "This is {hi:mcforQ}, ver 1.5."
	noi di as txt "Calculates Monte-Carlo p-values for extreme homogeneity between the individual"
	noi di as txt "studies' RR in a meta-analysis."
	noi di as txt "See help {help mcforQ} for more info."
	noi di ""

	if "`verbose'"!="noverbose" {  // how verbose the output is
		local noi "noi"
	}

	preserve // The initial file is preserved

	if `"`if'`in'"' != "" {
		keep `if' `in'
	}
 

	tempvar nExp cExp nCtrl cCtrl __has1zero __deadStudy  __nEvents __N1 __N2 prop1 prop2 

	local _n1 = word(`"`varlist'"', 1) // the variable for n1
	local _c1 = word(`"`varlist'"', 2) // the variable for c1
	local _n2 = word(`"`varlist'"', 3) // the variable for n2
	local _c2 = word(`"`varlist'"', 4) // the variable for c2
	g `__deadStudy' = ((floor(`_n1')==0 & floor(`_n2')==0) | (floor(`_c1')==0 & floor(`_c2')==0)) // only those with 2 zeros
	g `__has1zero' = ((`_n1'==0 | `_n2'==0 | `_c1'==0 | `_c2'==0) & `__deadStudy'==0)  // only those with 1 zero
	summ `__deadStudy'
	local deadStudy =`r(sum)'  // count the dead studies
	summ `__has1zero'
	local has1zero = `r(sum)'  // count the studies with 0 arms

	noi di as txt "{res:`deadStudy'} of {res:`r(N)'} studies have 0% vs 0% or 100% vs 100% arms"
	noi di as txt "{res:`has1zero'} of {res:`r(N)'} studies have only one arm with 0% or 100% counts"

	g `nExp' = floor(`_n1')
	g `cExp' = floor(`_c1')
	g `nCtrl' = floor(`_n2')
	g `cCtrl' = floor(`_c2')
	replace `nExp' = `nExp' + `ccone' if `__has1zero'==1  // continuity corrected values
	replace `cExp' = `cExp' + `ccone' if `__has1zero'==1
	replace `nCtrl' = `nCtrl' + `ccone' if `__has1zero'==1
	replace `cCtrl' = `cCtrl' + `ccone' if `__has1zero'==1
	replace `nExp' = `nExp' + `ccboth' if `__deadStudy'==1
	replace `cExp' = `cExp' + `ccboth' if `__deadStudy'==1
	replace `nCtrl' =`nCtrl' + `ccboth' if `__deadStudy'==1
	replace `cCtrl' = `cCtrl' + `ccboth' if `__deadStudy'==1

	`noi' metan `nExp' `cExp' `nCtrl' `cCtrl' , rr log nokeep fixed nointeger nograph  // calculate the meta-analysis
	local _logRR = `r(ES)'
	local _selogRR = `r(seES)'
	local Qobs = `r(het)' // the Qobs value
	local k = `r(df)' + 1  // the number of studies

	g  `__nEvents' = `nExp' + `nCtrl' 
	g  `__N1' = `nExp' + `cExp'
	g  `__N2' = `nCtrl' + `cCtrl' 

	return scalar k = `k'
	return scalar nEffective = `k' - `deadStudy'
	return scalar Qobs = `Qobs'

	/* set up the local macros for the studies */
	forvalues i=1/`k'{
		local nEvents`i' = `__nEvents' in `i'
		local N1`i' = `__N1' in `i'
		local N2`i' = `__N2' in `i'
	} // end forvalues loop

/************************************************************************/
/************************************************************************/
/************************************************************************/
/************************************************************************/

	clear

	set obs `sims'
	set seed `seed' // for Stata's uniform()

	if "`varyrr'"=="novaryrr" {
	    noi di as res "RRMH is considered fixed." 
	    g RRMH = exp(`_logRR')
	}
	else {
	    noi di as res "logRRMH is drawn from a Normal distribution."
	    g RRMH = exp((invnorm(uniform()) + `_logRR') * `_selogRR')
	}

	if "`varyprev'"=="novaryprev" {
		noi di as res "Overall events prevalence per study is fixed."
		forval i=1(1)`k' {
    		local seed1 = ceil(12345*uniform())  // to feed the seed to rndbinom.plugin
			local seed2 = ceil(23456*uniform())
      		g nEvents`i' = `nEvents`i''          // these have been continuity corrected
			
      		/* Because the RR may be from a random distribution, prop1 and pro2 
      		might be expelled outside the [0,1] interval, though this is not often. for this reason I 
      		bound them inside the interval.  This might mean a slight excess at the cutoff points.
      		I don't think this is important.  This happens only for randomly drawn RRs
      		*/
			
      		g prop2`i' = (nEvents`i'/(1+ RRMH * `N1`i''/`N2`i''))/`N2`i''
      		replace prop2`i' = 1 if prop2`i' > 1
      		replace prop2`i' = 0 if prop2`i' < 0
      
      		g prop1`i' = (nEvents`i' - `N2`i'' * prop2`i')/`N1`i''
      		replace prop1`i' = 1 if prop1`i' > 1
      		replace prop1`i' = 0 if prop1`i' < 0
      
      		drop nEvents`i'     // no need for this any more
      		g N1`i' = floor(`N1`i'')
      		g n1`i' = . 
		rndbinom prop1`i' N1`i' n1`i', seed(`seed1')
      		g c1`i' = N1`i' - n1`i'
      		drop N1`i'          // no need for this any more
      		g N2`i' = floor(`N2`i'')
      		g n2`i' = . 
      		rndbinom prop2`i' N2`i' n2`i', seed(`seed2')
      		g c2`i' = N2`i' - n2`i'
      		drop N2`i'          // no need for this any more
		}
	}
	else {
   		noi di as res "Overall events prevalence per study is drawn from a binomial distribution."
   		forval i=1(1)`k' {
      		local seed1 = ceil(12345*uniform())
      		local seed2 = ceil(23456*uniform())
      		local seed3 = ceil(34567*uniform())
      		g Ntotal`i' =  floor(`N1`i'' + `N2`i'')     // removes cc <.25 from total, no big deal
      		g propPrev`i' =  `nEvents`i'' / (`N1`i'' + `N2`i'')  // incorporates cc's
      		g nEvents`i' = .
      		rndbinom propPrev`i' Ntotal`i' nEvents`i', seed(`seed1')
      		drop Ntotal`i' propPrev`i'  // no need for these anymore
			
      		/* Because prev is drawn irrespectively of the odds ratio in this approach, prop1 and pro2 
      		might be expelled outside the [0,1] interval, though this is not often. for this reason I 
      		bound them inside the interval.  This might mean a slight excess at the cutoff points.
      		I don't think this is important.  
      		*/
			
      		g prop2`i' = (nEvents`i'/(1+ RRMH * `N1`i''/`N2`i''))/`N2`i''
      		replace prop2`i' = 1 if prop2`i' > 1
      		replace prop2`i' = 0 if prop2`i' < 0

      		g prop1`i' = (nEvents`i' - `N2`i'' * prop2`i')/`N1`i''
      		replace prop1`i' = 1 if prop1`i' > 1
      		replace prop1`i' = 0 if prop1`i' < 0
      
      		g N1`i' = floor(`N1`i'')
      		g n1`i' = . 
      		rndbinom prop1`i' N1`i' n1`i', seed(`seed2')
      		g c1`i' = N1`i' - n1`i'
      		drop N1`i'          // no need for this any more
      		g N2`i' = floor(`N2`i'')
      		g n2`i' = . 
      		rndbinom prop2`i' N2`i' n2`i', seed(`seed3')
      		g c2`i' = N2`i' - n2`i'
   	   		drop N2`i'          // no need for this any more      
   		}
	}


	forval i=1(1)`k' {

		g hasBothNZero`i' = 1 if n1`i'==0 & n2`i'==0
		replace hasBothNZero`i' = 1 if (c1`i'==0 & c2`i'==0)
		replace hasBothNZero`i' =0 if hasBothNZero`i'==.
		g hasOneNZero`i' =1 if (n1`i'==0|n2`i'==0) & !hasBothNZero
		replace hasOneNZero`i' =1 if (c1`i'==0|c2`i'==0) & !hasBothNZero
		replace hasOneNZero`i' =0 if hasOneNZero`i' ==.

		replace n1`i' = n1`i' + `ccboth' if hasBothNZero`i'
		replace n2`i' = n2`i' + `ccboth' if hasBothNZero`i'
		replace c1`i' = c1`i' + `ccboth' if hasBothNZero`i'
		replace c2`i' = c2`i' + `ccboth' if hasBothNZero`i'

		replace n1`i' = n1`i' + `ccone' if hasOneNZero`i'
		replace n2`i' = n2`i' + `ccone' if hasOneNZero`i'
		replace c1`i' = c1`i' + `ccone' if hasOneNZero`i'
		replace c2`i' = c2`i' + `ccone' if hasOneNZero`i'

		drop hasBothNZero`i' hasOneNZero`i'

		/* Now we do a Mantel-Haenszel meta-analysis */
		g RR`i' = (n1`i' * (n2`i' + c2`i'))/((n1`i' + c1`i') *n2`i')
		g weightRR`i' = n2`i'*(n1`i' + c1`i')/(n1`i' + c1`i' + n2`i' + c2`i') // weight for the effect size, not for Q
		g product`i' = RR`i' * weightRR`i'  // product for the effect size, not for Q
		g weightQ`i' = 1/(1/n1`i' + 1/n2`i' - 1/(n1`i' + c1`i') - 1/(n2`i' + c2`i')) // weight for Q 

		/* Useful Quantities */
		g P`i' = ((n1`i' + c1`i')* (n2`i' + c2`i') * (n1`i'+n2`i')-n1`i'*n2`i'*(n1`i' + c1`i' + n2`i'+ c2`i'))/(n1`i' + c1`i' + n2`i'+ c2`i')^2
		g R`i' = (n1`i'*(n2`i' + c2`i'))/(n1`i' + c1`i' + n2`i'+ c2`i')
		g S`i' = (n2`i'*(n1`i' + c1`i'))/(n1`i' + c1`i' + n2`i'+ c2`i')

		/* to use with egen below */
		local WhichProducts `"`WhichProducts' product`i'"'
		local WhichWeights `"`WhichWeights' weightRR`i'"'
		local WhichP `"`WhichP' P`i'"'
		local WhichR `"`WhichR' R`i'"'
		local WhichS `"`WhichS' S`i'"'
	}

	egen sumOfProducts = rsum(`WhichProducts')
	egen sumOfWeights = rsum(`WhichWeights')
	egen sumOfP = rsum(`WhichP')
	egen sumOfR = rsum(`WhichR')
	egen sumOfS = rsum(`WhichS')

	g logRRFixed_logRR = log(sumOfProducts / sumOfWeights)
	g logRRFixed_selogRR = sqrt(sumOfP/(sumOfR*sumOfS))

	forval i=1(1)`k' {
		g prodForQ`i' = weightQ`i' * (log(RR`i') - logRRFixed_logRR)^2
		local WhichProdForQ `"`WhichProdForQ' prodForQ`i'"'
	}

	egen Q = rsum(`WhichProdForQ')

	g ISquare = 1 - `k'/Q
	replace ISquare = 0 if ISquare<0

	if "`verbose'"!="noverbose" {
		noi di "min-max and quantiles (pN):"
		noi tabstat Q ISquare, stats(min p1 p5 p10 p50 p90 p95 p99 max) format(%9.4f)
	}
	count if Q <= `Qobs'
	local pExactQ = `r(N)'/`sims'
	return scalar pExactQ = `pExactQ'
	noi di ""
	noi di as txt "The observed value of {it:Q} was {res:`Qobs'}. Based on {res:`sims'} simulations,"
	noi di as txt "the left sided MonteCarlo p-Value is: p= {res:`pExactQ'} (for {it:Q <= Qobserved})"
	noi di ""

	drop sumOfProducts  // start dropping unneeded vars 
	drop sumOfWeights
	drop sumOfP
	drop sumOfR
	drop sumOfS
	forvalues i=1/`k'{
		drop product`i'
		drop weightRR`i'
		drop prodForQ`i'
		drop weightQ`i'
		drop P`i'
		drop S`i'
		drop R`i'
	} // end drop

	clear
	restore
}   // ends quietly
end



program define rndbinom
syntax varlist(numeric min=3 max=3) [in] [if] [ , seed(int -999) ]

local prob "`1'"
local N "`2'"
local r = word("`varlist'",3)

cap assert (`prob'<=1 & `prob' >=0) | `prob' >=.
if (_rc ==9) {
	noi di as error "The variable for the probability (1st in {it:varlist}) must have all non-missing"
	noi di as error "values in the [0,1] interval."
	noi di ""
	exit -1
}

cap assert `N'>0 & (mod(`N',1)==0 | `N' >=.)
if (_rc ==9) {
	noi di as error "The variable for total (2nd in {it:varlist}) must be a positive integer."
	noi di ""
	exit -1
}

if ("`c(os)'"=="Windows") {
	plugin call rndbinomi2_win `prob' `N' `r'  `if'`in' , `seed'
}
else if ("`c(os)'"=="MacOSX") {
	plugin call rndbinomi2_osx `prob' `N' `r'  `if'`in' , `seed'
}
else {
	plugin call rndbinomi2 `prob' `N' `r'  `if'`in' , `seed'
}
end




if ("`c(os)'"=="Windows") {
	program define rndbinomi2_win , plugin
}
else if ("`c(os)'"=="MacOSX") {
	program define rndbinomi2_osx , plugin
}
else {
	program define rndbinomi2 , plugin
}

