#include <stdlib.h>
#include <stplugin.h>
#include <math.h>
#define PI 3.141592654
#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

/*
The plugin arguments are Probability, NTotal, K (to fill in), seed
I implemented a portable but high quality random number generator from the 
numerical recipes in C book.
I implemented the binomial random number generator from the 
numerical recipes in C book.
*/
float bnldev (float pp, int n, long *idum);

STDLL stata_call(int argc, char *argv[]) {
    ST_uint4             j, k, n;
    ST_float             p;
    ST_double            z, z1, z2;
    ST_retcode           rc;
    char *               endp; 
    long 		 seed;
    long *               idum;

    
   // setting the seed
    if(argc) {
        seed = (long)strtod(argv[0], &endp); // one must initialize the random number
        if (seed >0) seed = -(labs(seed));   // generator with a negative integer
        if (seed==0) seed = -999;
	seed = (long) ((long int)seed);        // seed must be a negative integer
    }
    else {
       SF_error("Please specify a seed; I'm using -999 (the default)");
       seed = -999;
    }
    idum=0;
    idum= &seed;

    
    if (SF_nvars() != 3)  {
        return(102);
    }
    
    for(j=SF_in1(); j<= SF_in2(); j++) {
        if (SF_ifobs(j)) {
	    if(rc=SF_vdata(1,j,&z)) return(rc);
	    z1=z;
	    p=(float)z;
	    if(rc=SF_vdata(2,j,&z)) return(rc);
	    z2=z;
	    n=(int)z;
	    if(SF_is_missing(z1) | SF_is_missing(z2)) {
	        if(rc=SF_vstore(SF_nvars(),j,SV_missval)) return(rc);
	    } 
	    else {
	        k= (ST_uint4) bnldev(p,n,idum);
	        if(rc=SF_vstore(SF_nvars(),j,k)) return(rc);
	    }
	}

    }
    return(0);
}



float bnldev (float pp, int n, long *idum) {
	
	int j;
	static int nold=(-1);
	float am, em, g, angle, p, bnl, sq, t, y;
	static float pold=(-1.0), pc, plog, pclog, en, oldg;
	
	
	float gammaln (float xx) {
		double x, y, tmp, ser;
		static double cof[6]={76.18009172947146, -86.50532032941677,
			24.01409824083091,-1.231739572450155,
			0.1208650973866179e-2,-0.5395239384953e-5};
		int j;
	
		y=x=xx;
		tmp = x+5.5;
		tmp -= (x+0.5)*log(tmp);
		ser=1.000000000190015;
		for (j=0;j<=5;j++) ser += cof[j]/++y;
		return -tmp+log(2.5066282746310005*ser/x);
	}

	
	float ran1(long * idum) {
		int j; 
		long k;
		static long iy=0;
		static long iv[NTAB];
		float temp;
	
		if (*idum <=0 || !iy ) {
			if (-(*idum) < 1) *idum=1;
			else *idum = -(*idum);
			for (j=NTAB+7;j>=0;j--) {
				k=(*idum)/IQ;
				*idum=IA*(*idum-k*IQ)-IR*k;
				if (*idum<0) *idum +=IM;
				if (j<NTAB) iv[j] = *idum;
			}
			iy = iv[0];
		}
		k=(*idum)/IQ;
		*idum=IA*(*idum-k*IQ)-IR*k;
		if (*idum<0) *idum += IM;
		j=iy/NDIV;
		iy=iv[j]; 
		iv[j]=*idum;
	
		if ((temp=AM*iy) > RNMX) return RNMX;
		else return temp;
	}

	p=(pp<=0.5 ? pp: 1.0-pp);
	am=n*p; 
	if (n<25) {
		bnl=0.0;
		for (j=1;j<=n;j++)
			if (ran1(idum) < p) ++bnl;
	} else if (am <1.0) {
		g=exp(-am);
		t=1.0;
		for (j=0;j<=n;j++) {
			t *= ran1(idum);
			if (t<g) break;
		}
		bnl=(j <=n ? j : n);
	} else {
		if (n != nold) {
			en = n;
			oldg=gammaln(en+1.0);
			nold =n;
		} if ( p!= pold) {
			pc=1.0-p;
			plog=log(p);
			pclog=log(pc);
			pold=p;
		}
		sq=sqrt(2.0*am*pc);
		do {
			do {
				angle=PI*ran1(idum);
				y=tan(angle);
				em=sq*y+am;
			} while (em <0.0 || em >= (en + 1.0));
			em=floor(em);
			t=1.2*sq*(1.0+y*y)*exp(oldg-gammaln(em + 1.0)
				-gammaln(en-em+1.0) + em*plog+(en-em)*pclog);
		} while (ran1(idum) > t);
		bnl=em;
	}
	if (p !=pp) bnl=n-bnl;
	return bnl;
} 


/*
// The following code if compiled against the GSL library.
// 25% slower, but far more robust rand generator.


STDLL stata_call(int argc, char *argv[]) {
    ST_uint4             j, k, n , seed; 
    ST_float             p;
    ST_double            z, z1, z2;
    ST_retcode           rc;
    char *               endp; 
    const gsl_rng_type * T; 
    gsl_rng *            r;

    T=gsl_rng_default;   // random number generator is a die hard generator, good
    r=gsl_rng_alloc(T);  
   
    if(argc) {
    seed = (int)strtod(argv[0], &endp);
    }
    else {
       SF_error("Please specify a seed; I'm using -999 (the default)");
       seed = -999;
    }
    gsl_rng_set(r, seed); // set seed
    
    if (SF_nvars() != 3)  {
        return(102);
    }
    
    for(j=SF_in1(); j<= SF_in2(); j++) {
        if (SF_ifobs(j)) {
	    if(rc=SF_vdata(1,j,&z)) return(rc);
	    z1=z;
	    p=(float)z;
	    if(rc=SF_vdata(2,j,&z)) return(rc);
	    z2=z;
	    n=(int)z;
	    if(SF_is_missing(z1) | SF_is_missing(z2)) {
	        if(rc=SF_vstore(SF_nvars(),j,SV_missval)) return(rc);
	    } 
	    else {
	        k= gsl_ran_binomial(r,p,n);
	        if(rc=SF_vstore(SF_nvars(),j,k)) return(rc);
	    }
	}

    }
    gsl_rng_free(r); // free memory occupied by r
    return(0);
}
*/

